package com.fs.stats;

import com.fs.common.core.domain.R;
import com.fs.common.core.page.TableDataInfo;
import com.fs.company.mapper.CompanyMapper;
import com.fs.company.service.ICompanyService;
import com.fs.company.vo.CompanyDataVO;
import com.fs.sop.service.IQwSopService;
import com.fs.sop.vo.QwSopTask;
import com.fs.statis.domain.FsStatisPeriodWatch;
import com.fs.statis.domain.FsStatisSalerWatch;
import com.fs.statis.dto.StatsWatchLogPageListDTO;
import com.fs.statis.service.FsStatisEveryDayWatchService;
import com.fs.statis.service.FsStatisPeriodWatchService;
import com.fs.statis.service.FsStatisSalerWatchService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 看课统计接口
 */
@RestController
@RequestMapping("/stats")
@AllArgsConstructor
public class SalesWatchStatisController {

    @Autowired
    private FsStatisSalerWatchService fsStatisSalerWatchService;

    @Autowired
    private FsStatisPeriodWatchService fsStatisPeriodWatchService;

    @Autowired
    private FsStatisEveryDayWatchService fsStatisEveryDayWatchService;

    @Autowired
    private IQwSopService qwSopService;

    /**
     * 销售完播统计查询
     * @param param param
     * @return R
     */
    @PostMapping("/seller/pageList")
    public R sellerQueryList(@RequestBody StatsWatchLogPageListDTO param){
        List<FsStatisSalerWatch> list = fsStatisSalerWatchService.queryList(param);
        return R.ok().put("data",list);
    }

    /**
     * 训练营完播统计查询
     * @param param param
     * @return R
     */
    @PostMapping("/period/pageList")
    public R periodQueryList(@RequestBody StatsWatchLogPageListDTO param){
        List<FsStatisPeriodWatch> list = fsStatisPeriodWatchService.queryList(param);
        return R.ok().put("data", list);
    }

    /**
     * 每日完播统计查询
     * @param param param
     * @return R
     */
    @PostMapping("/everyDay/pageList")
    public R everyDayQueryList(@RequestBody StatsWatchLogPageListDTO param){
        List<FsStatisSalerWatch> list = fsStatisEveryDayWatchService.queryList(param);
        return R.ok().put("data", list);
    }

    /**
     * 获取SOP任务数据
     * @return
     */
    @GetMapping("/sopTaskData")
    public R getSOPTaskData(){
        List<QwSopTask> qwSopTaskList = qwSopService.getQwSopTaskList();

        return R.ok().put("data",qwSopTaskList);
    }

}
