package com.fs.web.controller.system;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.fs.common.core.domain.R;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fs.common.annotation.Log;
import com.fs.common.core.controller.BaseController;
import com.fs.common.core.domain.AjaxResult;
import com.fs.common.enums.BusinessType;
import com.fs.system.domain.SysKeyword;
import com.fs.system.service.ISysKeywordService;
import com.fs.common.utils.poi.ExcelUtil;
import com.fs.common.core.page.TableDataInfo;

/**
 * 系统关键字Controller
 *
 * @author fs
 * @date 2025-05-14
 */
@RestController
@RequestMapping("/system/keyword")
public class SysKeywordController extends BaseController
{
    @Autowired
    private ISysKeywordService sysKeywordService;

    /**
     * 查询系统关键字列表
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:list')")
    @GetMapping("/list")
    public R list(SysKeyword sysKeyword)
    {
//        startPage();
        PageHelper.startPage(sysKeyword.getPageNum(), sysKeyword.getPageSize());
        List<SysKeyword> list = sysKeywordService.selectSysKeywordList(sysKeyword);

        PageInfo<SysKeyword> pageInfo = new PageInfo<>(list);
        Map<String, Object> result = new HashMap<>();
        result.put("rows", pageInfo.getList());
        result.put("total", pageInfo.getTotal());
        return R.ok(result);
    }

    /**
     * 导出系统关键字列表
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:export')")
    @Log(title = "系统关键字", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(SysKeyword sysKeyword)
    {
        List<SysKeyword> list = sysKeywordService.selectSysKeywordList(sysKeyword);
        ExcelUtil<SysKeyword> util = new ExcelUtil<SysKeyword>(SysKeyword.class);
        return util.exportExcel(list, "系统关键字数据");
    }

    /**
     * 获取系统关键字详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:query')")
    @GetMapping(value = "/{keywordId}")
    public AjaxResult getInfo(@PathVariable("keywordId") Long keywordId)
    {
        return AjaxResult.success(sysKeywordService.selectSysKeywordById(keywordId));
    }

    /**
     * 新增系统关键字
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:add')")
    @Log(title = "系统关键字", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SysKeyword sysKeyword)
    {
        return toAjax(sysKeywordService.insertSysKeyword(sysKeyword));
    }

    /**
     * 修改系统关键字
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:edit')")
    @Log(title = "系统关键字", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SysKeyword sysKeyword)
    {
        return toAjax(sysKeywordService.updateSysKeyword(sysKeyword));
    }

    /**
     * 删除系统关键字
     */
    @PreAuthorize("@ss.hasPermi('system:keyword:remove')")
    @Log(title = "系统关键字", businessType = BusinessType.DELETE)
	@DeleteMapping("/{keywordIds}")
    public AjaxResult remove(@PathVariable Long[] keywordIds)
    {
        return toAjax(sysKeywordService.deleteSysKeywordByIds(keywordIds));
    }
}
