package com.fs.company.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.fs.common.exception.base.BaseException;
import com.fs.common.utils.DateUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fs.company.domain.Company;
import com.fs.company.service.ICompanyService;
import com.fs.course.config.CourseConfig;
import com.fs.system.service.ISysConfigService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.company.mapper.CompanyRedPackageLogsMapper;
import com.fs.company.domain.CompanyRedPackageLogs;
import com.fs.company.service.ICompanyRedPackageLogsService;

/**
 * 公司红包余额日志Service业务层处理
 * 
 * @author 吴树波
 * @date 2025-04-15
 */
@Service
@AllArgsConstructor
public class CompanyRedPackageLogsServiceImpl extends ServiceImpl<CompanyRedPackageLogsMapper, CompanyRedPackageLogs> implements ICompanyRedPackageLogsService {

    private final ISysConfigService configService;
    private final ICompanyService companyService;
    /**
     * 查询公司红包余额日志
     * 
     * @param id 公司红包余额日志主键
     * @return 公司红包余额日志
     */
    @Override
    public CompanyRedPackageLogs selectCompanyRedPackageLogsById(Long id)
    {
        return baseMapper.selectCompanyRedPackageLogsById(id);
    }

    /**
     * 查询公司红包余额日志列表
     * 
     * @param companyRedPackageLogs 公司红包余额日志
     * @return 公司红包余额日志
     */
    @Override
    public List<CompanyRedPackageLogs> selectCompanyRedPackageLogsList(CompanyRedPackageLogs companyRedPackageLogs)
    {
        return baseMapper.selectCompanyRedPackageLogsList(companyRedPackageLogs);
    }

    /**
     * 新增公司红包余额日志
     * 
     * @param companyRedPackageLogs 公司红包余额日志
     * @return 结果
     */
    @Override
    public int insertCompanyRedPackageLogs(CompanyRedPackageLogs companyRedPackageLogs)
    {
        companyRedPackageLogs.setCreateTime(DateUtils.getNowDate());
        return baseMapper.insertCompanyRedPackageLogs(companyRedPackageLogs);
    }

    /**
     * 修改公司红包余额日志
     * 
     * @param companyRedPackageLogs 公司红包余额日志
     * @return 结果
     */
    @Override
    public int updateCompanyRedPackageLogs(CompanyRedPackageLogs companyRedPackageLogs)
    {
        companyRedPackageLogs.setUpdateTime(DateUtils.getNowDate());
        return baseMapper.updateCompanyRedPackageLogs(companyRedPackageLogs);
    }

    /**
     * 批量删除公司红包余额日志
     * 
     * @param ids 需要删除的公司红包余额日志主键
     * @return 结果
     */
    @Override
    public int deleteCompanyRedPackageLogsByIds(Long[] ids)
    {
        return baseMapper.deleteCompanyRedPackageLogsByIds(ids);
    }

    /**
     * 删除公司红包余额日志信息
     * 
     * @param id 公司红包余额日志主键
     * @return 结果
     */
    @Override
    public int deleteCompanyRedPackageLogsById(Long id)
    {
        return baseMapper.deleteCompanyRedPackageLogsById(id);
    }

    @Override
    public void recharge(CompanyRedPackageLogs entity) {
        if(entity.getUpMoney() == null) throw new BaseException("充值金额不能为空");
        if(entity.getUpMoney().doubleValue() < 1) throw new BaseException("充值金额不能小于1元");
        entity.setHandlPri(getMoneyPri());
        entity.setHandlMoney(entity.getUpMoney().multiply(entity.getHandlPri().divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP)));
        entity.setMoney(entity.getUpMoney().subtract(entity.getHandlMoney()));
        entity.setType(0);
        entity.setOperateType(0);
        save(entity);
    }

    @Override
    public BigDecimal getMoneyPri() {
        String json = configService.selectConfigByKey("course.config");
        CourseConfig config = JSON.parseObject(json, CourseConfig.class);
        return config.getMoneyPri();
    }

    @Override
    public void rew(CompanyRedPackageLogs entity) {
        CompanyRedPackageLogs byId = getById(entity.getId());
        if(byId != null) {
            byId.setStatus(entity.getStatus());
            byId.setRemark(entity.getRemark());
            byId.setRejectMemo(entity.getRejectMemo());
            byId.setRejectImages(entity.getRejectImages());
            updateById(byId);
            if(byId.getStatus() == 1){
                companyService.updateMoney(byId.getCompanyId(), byId.getMoney());
            }
        }
    }
}
