package com.fs.sop.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fs.common.annotation.DataSource;
import com.fs.common.enums.DataSourceType;
import com.fs.sop.domain.QwSopTempRules;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * sop任务模板规则Mapper接口
 *
 * @author fs
 * @date 2025-02-06
 */
@DataSource(DataSourceType.SOP)
public interface QwSopTempRulesMapper extends BaseMapper<QwSopTempRules>{
    /**
     * 查询sop任务模板规则
     *
     * @param id sop任务模板规则主键
     * @return sop任务模板规则
     */
    QwSopTempRules selectQwSopTempRulesById(String id);

    /**
     * 查询sop任务模板规则列表
     *
     * @param qwSopTempRules sop任务模板规则
     * @return sop任务模板规则集合
     */
    List<QwSopTempRules> selectQwSopTempRulesList(QwSopTempRules qwSopTempRules);

    /**
     * 新增sop任务模板规则
     *
     * @param qwSopTempRules sop任务模板规则
     * @return 结果
     */
    int insertQwSopTempRules(QwSopTempRules qwSopTempRules);

    /**
     * 修改sop任务模板规则
     *
     * @param qwSopTempRules sop任务模板规则
     * @return 结果
     */
    int updateQwSopTempRules(QwSopTempRules qwSopTempRules);

    /**
     * 删除sop任务模板规则
     *
     * @param id sop任务模板规则主键
     * @return 结果
     */
    int deleteQwSopTempRulesById(String id);

    /**
     * 批量删除sop任务模板规则
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteQwSopTempRulesByIds(String[] ids);

    @Select("select * from qw_sop_temp_rules where day_num is null ")
    List<QwSopTempRules> rulesNull();



    void deleteByTempId(@Param("id") String id);
    void insertBatch(@Param("list") List<QwSopTempRules> list);

    List<QwSopTempRules> listByTempId(@Param("id") String id);

    List<QwSopTempRules> listByTempIdAndNameAndDayNum(@Param("id") String id, @Param("dayNum") Integer dayNum);

    void deleteByIdList(@Param("ids") List<String> ids);

}
