package com.fs.store.mapper;

import java.util.List;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fs.store.domain.FsUserCompanyUser;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 微信用户和销售关系Mapper接口
 *
 * @author fs
 * @date 2025-05-09
 */
public interface FsUserCompanyUserMapper extends BaseMapper<FsUserCompanyUser>{
    /**
     * 查询微信用户和销售关系
     *
     * @param id 微信用户和销售关系主键
     * @return 微信用户和销售关系
     */
    FsUserCompanyUser selectFsUserCompanyUserById(Long id);

    /**
     * 查询微信用户和销售关系列表
     *
     * @param fsUserCompanyUser 微信用户和销售关系
     * @return 微信用户和销售关系集合
     */
    List<FsUserCompanyUser> selectFsUserCompanyUserList(FsUserCompanyUser fsUserCompanyUser);

    /**
     * 新增微信用户和销售关系
     *
     * @param fsUserCompanyUser 微信用户和销售关系
     * @return 结果
     */
    int insertFsUserCompanyUser(FsUserCompanyUser fsUserCompanyUser);

    /**
     * 修改微信用户和销售关系
     *
     * @param fsUserCompanyUser 微信用户和销售关系
     * @return 结果
     */
    int updateFsUserCompanyUser(FsUserCompanyUser fsUserCompanyUser);

    /**
     * 删除微信用户和销售关系
     *
     * @param id 微信用户和销售关系主键
     * @return 结果
     */
    int deleteFsUserCompanyUserById(Long id);

    /**
     * 批量删除微信用户和销售关系
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteFsUserCompanyUserByIds(Long[] ids);

    /**
     * 获取当前销售的所有重粉会员
     * @param companyUserId
     * @return
     */
    @Select("select user_id from fs_user_company_user where company_user_id = #{userId} and is_repeat_fans = 1 ")
    List<FsUserCompanyUser> selectRepeatUser(Long companyUserId);

    /**
     * 获取会员的重粉的所属销售
     * @param userIds 用户会员ids
     * @return
     */
    List<FsUserCompanyUser> selectRepeatCompanyUserName(@Param("userIds") List<Long> userIds);
}
