package com.fs.system.service.impl;

import java.util.List;
import com.fs.common.utils.DateUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.system.mapper.SysKeywordMapper;
import com.fs.system.domain.SysKeyword;
import com.fs.system.service.ISysKeywordService;

/**
 * 系统关键字Service业务层处理
 * 
 * @author fs
 * @date 2025-05-14
 */
@Service
public class SysKeywordServiceImpl extends ServiceImpl<SysKeywordMapper, SysKeyword> implements ISysKeywordService {

    /**
     * 查询系统关键字
     * 
     * @param keywordId 系统关键字主键
     * @return 系统关键字
     */
    @Override
    public SysKeyword selectSysKeywordById(Long keywordId)
    {
        return baseMapper.selectSysKeywordById(keywordId);
    }

    /**
     * 查询系统关键字列表
     * 
     * @param sysKeyword 系统关键字
     * @return 系统关键字
     */
    @Override
    public List<SysKeyword> selectSysKeywordList(SysKeyword sysKeyword)
    {
        return baseMapper.selectSysKeywordList(sysKeyword);
    }

    /**
     * 新增系统关键字
     * 
     * @param sysKeyword 系统关键字
     * @return 结果
     */
    @Override
    public int insertSysKeyword(SysKeyword sysKeyword)
    {
        sysKeyword.setCreateTime(DateUtils.getNowDate());
        return baseMapper.insertSysKeyword(sysKeyword);
    }

    /**
     * 修改系统关键字
     * 
     * @param sysKeyword 系统关键字
     * @return 结果
     */
    @Override
    public int updateSysKeyword(SysKeyword sysKeyword)
    {
        sysKeyword.setUpdateTime(DateUtils.getNowDate());
        return baseMapper.updateSysKeyword(sysKeyword);
    }

    /**
     * 批量删除系统关键字
     * 
     * @param keywordIds 需要删除的系统关键字主键
     * @return 结果
     */
    @Override
    public int deleteSysKeywordByIds(Long[] keywordIds)
    {
        return baseMapper.deleteSysKeywordByIds(keywordIds);
    }

    /**
     * 删除系统关键字信息
     * 
     * @param keywordId 系统关键字主键
     * @return 结果
     */
    @Override
    public int deleteSysKeywordById(Long keywordId)
    {
        return baseMapper.deleteSysKeywordById(keywordId);
    }
}
